@@ -1,9 +1,17 @@
+from box import Box, BoxList
 from restfly.endpoint import APIEndpoint
 
-from pyzscaler.utils import snake_to_camel
+from pyzscaler.utils import Iterator, convert_keys, snake_to_camel
 
 
 class PolicySetsAPI(APIEndpoint):
+    POLICY_MAP = {
+        "access": "ACCESS_POLICY",
+        "timeout": "TIMEOUT_POLICY",
+        "client_forwarding": "CLIENT_FORWARDING_POLICY",
+        "siem": "SIEM_POLICY",
+    }
+
     @staticmethod
     def _create_conditions(conditions):
         """
@@ -34,62 +42,66 @@ class PolicySetsAPI(APIEndpoint):
 
         return template
 
-    def list_policies(self, policy_type: str):
+    def get_policy(self, policy_type: str) -> Box:
         """
         Returns the policy and rule sets for the given policy type.
 
         Args:
             policy_type (str): The type of policy to be returned. Accepted values are:
 
-                 |  `access`
-                 |  `timeout`
-                 |  `client_forwarding`
+                 |  ``access`` - returns the Access Policy
+                 |  ``timeout`` - returns the Timeout Policy
+                 |  ``client_forwarding`` - returns the Client Forwarding Policy
+                 |  ``siem`` - returns the SIEM Policy
 
         Returns:
-            :obj:`dict`: Resource record of the specified policy.
+            :obj:`Box`: The resource record of the specified policy type.
 
         Examples:
             Request the specified Policy.
 
-            >>> pprint(zpa.policies.list_policies('access'))
+            >>> pprint(zpa.policies.get_policy('access'))
 
         """
-        # Rename policy type to names that ZPA API is expecting.
-        if policy_type == "access":
-            policy_type = "global"
-        elif policy_type == "timeout":
-            policy_type = "reauth"
-        elif policy_type == "client_forwarding":
-            policy_type = "bypass"
+        # Map the simplified policy_type name to the name expected by the Zscaler API
+        mapped_policy_type = self.POLICY_MAP.get(policy_type, None)
 
-        return self._get(f"policySet/{policy_type}")
+        # If the user provided an incorrect name, raise an error
+        if not mapped_policy_type:
+            raise ValueError(
+                f"Incorrect policy type provided: {policy_type}\n "
+                f"Policy type must be 'access', 'timeout', 'client_forwarding' or 'siem'."
+            )
 
-    def get_rule(self, policy_type: str, rule_id: str):
+        return self._get(f"policySet/policyType/{mapped_policy_type}")
+
+    def get_rule(self, policy_type: str, rule_id: str) -> Box:
         """
         Returns the specified policy rule.
 
         Args:
             policy_type (str): The type of policy to be returned. Accepted values are:
 
-                 |  `access`
-                 |  `timeout`
-                 |  `client_forwarding`
+                 |  ``access``
+                 |  ``timeout``
+                 |  ``client_forwarding``
+                 |  ``siem``
             rule_id (str): The unique identifier for the policy rule.
 
         Returns:
-            :obj:`dict`: The resource record for the requested rule.
+            :obj:`Box`: The resource record for the requested rule.
 
         Examples:
-            >>> policy_rule = zpa.policies.get_rule(policy_id='234234234344',
-            ...    rule_id='23456546546456')
+            >>> policy_rule = zpa.policies.get_rule(policy_id='99999',
+            ...    rule_id='88888')
 
         """
         # Get the policy id for the supplied policy_type
-        _policy_id = self.list_policies(policy_type).id
+        policy_id = self.get_policy(policy_type).id
 
-        return self._get(f"policySet/{_policy_id}/rule/{rule_id}")
+        return self._get(f"policySet/{policy_id}/rule/{rule_id}")
 
-    def list_rules(self, policy_type: str):
+    def list_rules(self, policy_type: str, **kwargs) -> BoxList:
         """
         Returns policy rules for a given policy type.
 
@@ -97,9 +109,9 @@ class PolicySetsAPI(APIEndpoint):
             policy_type (str):
                 The policy type. Accepted values are:
 
-                |  `access` - returns Access Policy rules
-                |  `timeout` - returns Timeout Policy rules
-                |  `client_forwarding` - returns Client Forwarding Policy rules
+                |  ``access`` - returns Access Policy rules
+                |  ``timeout`` - returns Timeout Policy rules
+                |  ``client_forwarding`` - returns Client Forwarding Policy rules
 
         Returns:
             :obj:`list`: A list of all policy rules that match the requested type.
@@ -110,17 +122,19 @@ class PolicySetsAPI(APIEndpoint):
 
         """
 
-        # Rename policy type to names that ZPA API is expecting.
-        if policy_type == "access":
-            policy_type = "GLOBAL_POLICY"
-        elif policy_type == "timeout":
-            policy_type = "REAUTH_POLICY"
-        elif policy_type == "client_forwarding":
-            policy_type = "BYPASS_POLICY"
+        # Map the simplified policy_type name to the name expected by the Zscaler API
+        mapped_policy_type = self.POLICY_MAP.get(policy_type, None)
 
-        return self._get(f"policySet/rules/policyType/{policy_type}").list
+        # If the user provided an incorrect name, raise an error
+        if not mapped_policy_type:
+            raise ValueError(
+                f"Incorrect policy type provided: {policy_type}\n "
+                f"Policy type must be 'access', 'timeout', 'client_forwarding' or 'siem'."
+            )
 
-    def delete_rule(self, policy_type: str, rule_id: str):
+        return BoxList(Iterator(self._api, f"policySet/rules/policyType/{mapped_policy_type}", **kwargs))
+
+    def delete_rule(self, policy_type: str, rule_id: str) -> int:
         """
         Deletes the specified policy rule.
 
@@ -128,27 +142,28 @@ class PolicySetsAPI(APIEndpoint):
             policy_type (str):
                 The type of policy the rule belongs to. Accepted values are:
 
-                 |  `access`
-                 |  `timeout`
-                 |  `client_forwarding`
+                 |  ``access``
+                 |  ``timeout``
+                 |  ``client_forwarding``
+                 |  ``siem``
             rule_id (str):
                 The unique identifier for the policy rule.
 
         Returns:
-            :obj:`str`: The response code for the operation.
+            :obj:`int`: The response code for the operation.
 
         Examples:
-            >>> zpa.policies.delete_rule(policy_id='2342567567355535',
-            ...    rule_id='23322646655778')
+            >>> zpa.policies.delete_rule(policy_id='99999',
+            ...    rule_id='88888')
 
         """
 
         # Get policy id for specified policy type
-        _policy_id = self.list_policies(policy_type).id
+        policy_id = self.get_policy(policy_type).id
 
-        return self._delete(f"policySet/{_policy_id}/rule/{rule_id}")
+        return self._delete(f"policySet/{policy_id}/rule/{rule_id}").status_code
 
-    def add_access_rule(self, name: str, action: str, **kwargs):
+    def add_access_rule(self, name: str, action: str, **kwargs) -> Box:
         """
         Add a new Access Policy rule.
 
@@ -161,8 +176,8 @@ class PolicySetsAPI(APIEndpoint):
             action (str):
                 The action for the policy. Accepted values are:
 
-                |  `allow`
-                |  `deny`
+                |  ``allow``
+                |  ``deny``
             **kwargs:
                 Optional keyword args.
 
@@ -175,18 +190,18 @@ class PolicySetsAPI(APIEndpoint):
 
                 .. code-block:: python
 
-                    [('app', 'id', '926196382959075416'),
-                    ('app', 'id', '926196382959075417'),
-                    ('app_group', 'id', '926196382959075332),
+                    [('app', 'id', '99999'),
+                    ('app', 'id', '88888'),
+                    ('app_group', 'id', '77777),
                     ('client_type', 'zpn_client_type_exporter', 'zpn_client_type_zapp'),
-                    ('trusted_network', 'b15e4cad-fa6e-8182-9fc3-8125ee6a65e1', True)]
+                    ('trusted_network', 'xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxx', True)]
             custom_msg (str):
                 A custom message.
             description (str):
                 A description for the rule.
 
         Returns:
-            :obj:`dict`: The resource record of the newly created access policy rule.
+            :obj:`Box`: The resource record of the newly created access policy rule.
 
         """
 
@@ -198,15 +213,15 @@ class PolicySetsAPI(APIEndpoint):
         }
 
         # Get the policy id of the provided policy type for the URL.
-        _policy_id = self.list_policies("access").id
+        policy_id = self.get_policy("access").id
 
         # Add optional parameters to payload
         for key, value in kwargs.items():
             payload[snake_to_camel(key)] = value
 
-        return self._post(f"policySet/{_policy_id}/rule", json=payload)
+        return self._post(f"policySet/{policy_id}/rule", json=payload)
 
-    def add_timeout_rule(self, name: str, **kwargs):
+    def add_timeout_rule(self, name: str, **kwargs) -> Box:
         """
         Add a new Timeout Policy rule.
 
@@ -243,7 +258,7 @@ class PolicySetsAPI(APIEndpoint):
                 The re-authentication timeout value in seconds.
 
         Returns:
-            :obj:`dict`: The resource record of the newly created Timeout Policy rule.
+            :obj:`Box`: The resource record of the newly created Timeout Policy rule.
 
         """
 
@@ -255,7 +270,7 @@ class PolicySetsAPI(APIEndpoint):
         }
 
         # Get the policy id of the provided policy type for the URL.
-        _policy_id = self.list_policies("timeout").id
+        _policy_id = self.get_policy("timeout").id
 
         # Use specified timeouts or default to UI values
         payload["reauthTimeout"] = kwargs.get("re_auth_timeout", 172800)
@@ -267,7 +282,7 @@ class PolicySetsAPI(APIEndpoint):
 
         return self._post(f"policySet/{_policy_id}/rule", json=payload)
 
-    def add_client_forwarding_rule(self, name: str, action: str, **kwargs):
+    def add_client_forwarding_rule(self, name: str, action: str, **kwargs) -> Box:
         """
         Add a new Client Forwarding Policy rule.
 
@@ -281,9 +296,9 @@ class PolicySetsAPI(APIEndpoint):
             action (str):
                 The action for the policy. Accepted values are:
 
-                |  `intercept`
-                |  `intercept_accessible`
-                |  `bypass`
+                |  ``intercept``
+                |  ``intercept_accessible``
+                |  ``bypass``
             **kwargs:
                 Optional keyword args.
 
@@ -307,7 +322,7 @@ class PolicySetsAPI(APIEndpoint):
                 A description for the rule.
 
         Returns:
-            :obj:`dict`: The resource record of the newly created Client Forwarding Policy rule.
+            :obj:`Box`: The resource record of the newly created Client Forwarding Policy rule.
 
         """
 
@@ -319,15 +334,15 @@ class PolicySetsAPI(APIEndpoint):
         }
 
         # Get the policy id of the provided policy type for the URL.
-        _policy_id = self.list_policies("client_forwarding").id
+        policy_id = self.get_policy("client_forwarding").id
 
         # Add optional parameters to payload
         for key, value in kwargs.items():
             payload[snake_to_camel(key)] = value
 
-        return self._post(f"policySet/{_policy_id}/rule", json=payload)
+        return self._post(f"policySet/{policy_id}/rule", json=payload)
 
-    def update_rule(self, policy_type: str, rule_id: str, **kwargs):
+    def update_rule(self, policy_type: str, rule_id: str, **kwargs) -> Box:
         """
         Update an existing policy rule
 
@@ -335,9 +350,9 @@ class PolicySetsAPI(APIEndpoint):
             policy_type (str):
                 The policy type. Accepted values are:
 
-                 |  `access`
-                 |  `timeout`
-                 |  `client_forwarding`
+                 |  ``access``
+                 |  ``timeout``
+                 |  ``client_forwarding``
             rule_id (str):
                 The unique identifier for the rule to be updated.
             **kwargs:
@@ -363,21 +378,22 @@ class PolicySetsAPI(APIEndpoint):
                 A description for the rule.
 
         Returns:
-            :obj:`str`: The response code for the operation.
+            :obj:`Box`: The updated policy-rule resource record.
 
         Examples:
             Updates the name only for an Access Policy rule:
 
-            >>> zpa.policies.update_rule('access', '72057615512764594', name='new_rule_name')
+            >>> zpa.policies.update_rule('access', '99999', name='new_rule_name')
 
             Updates the action only for a Client Forwarding Policy rule:
 
-            >>> zpa.policies.update_rule('client_forwarding', '72057615512764595', action='BYPASS')
+            >>> zpa.policies.update_rule('client_forwarding', '888888', action='BYPASS')
 
         """
+        # Get policy id for specified policy type
+        policy_id = self.get_policy(policy_type).id
 
-        current_rule = self.get_rule(policy_type, rule_id)
-        payload = {snake_to_camel(k): v for k, v in current_rule.items()}
+        payload = convert_keys(self.get_rule(policy_type, rule_id))
 
         # Add optional parameters to payload
         for key, value in kwargs.items():
@@ -386,8 +402,7 @@ class PolicySetsAPI(APIEndpoint):
             else:
                 payload[snake_to_camel(key)] = value
 
-        return self._put(
-            f"policySet/{current_rule.policySetId}/rule/{rule_id}",
-            json=payload,
-            box=False,
-        ).status_code
+        resp = self._put(f"policySet/{policy_id}/rule/{rule_id}", json=payload, box=False).status_code
+
+        if resp == 204:
+            return self.get_rule(policy_type, rule_id)
